mod agcd;
mod bkz;
mod deep_lll;
mod file;
mod lll;
mod matrix;
mod utils;
mod vector;

use crate::agcd::agcd;
use crate::file::parse_file;
use clap::{Parser, Subcommand};
use std::path::{Path, PathBuf};

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
#[command(propagate_version = true)]
struct Cli {
    #[command(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    /// approximate GCD
    Agcd {
        /// (default './input.txt')
        path: Option<PathBuf>,
        /// Algorithm variant to use (default: 0)
        #[arg(short = 'a', long, default_value_t = 0u8)]
        algorithm: u8,
    },
}

fn main() -> std::io::Result<()> {
    let cli = Cli::parse();

    match &cli.command {
        Commands::Agcd { path, algorithm } => {
            let path = path.as_deref().unwrap_or(Path::new("./input.txt"));
            let input = parse_file(path)?;

            agcd(input.numbers, input.noise_bits, *algorithm);
        }
    }
    Ok(())
}
